;;; moirai-core.el -*- lexical-binding: t; -*-

;;; superclass
(defclass moirai-strand ()
  ((display-alist
    :initarg :display-alist
    :documentation
    "Alist of conditionals actions for `display-buffer',
It is the same format that `display-buffer-alist'.")
   (window
    :initarg :window
    :initform nil
    :documentation
    "The window targeted by the strand.")
   (buffer
    :initarg :buffer
    :initform nil
    :documentation
    "The actual buffer of that strand.")
   (displayed
    :initarg :displayed
    :initform nil
    :documentation
    "Is that strand displayed?")
   (prev-buffers-predicate
    :initarg :prev-buffers-predicate
    :initform nil
    :documentation
    "The predicate used to set the prev-buffers list."))
  :abtract t
  :documentation "A superclass for all the moirai strands.")

;;; utilities
(defmacro moirai-ref (strand-name ref &optional frame)
  "Access the slot REF of the object STRAND-NAME in FRAME."
  `(oref (frame-parameter ,frame ,strand-name) ,ref))

(defun morai-get-or-create (class strand-name &optional frame &rest initargs)
  "Get the object STRAND-NAME based on CLASS in FRAME parameters or creates it with INITARGS,
Always return the object."
  (if-let ((strand (frame-parameter frame strand-name)))
      strand
    (set-frame-parameter frame strand-name
                         (funcall class initargs))
    (frame-parameter frame strand-name)))

;;; methods
(cl-defmethod morai-build-prev-buffer-history ((strand moirai-strand) window)
  "Build the `window-prev-buffers' list for the object STRAND and WINDOW."
  (let* ((buffer (window-buffer window))
         (predicate (oref strand :prev-buffers-predicate))
         (actual-prev-buffers (window-prev-buffers window))
         (saved-prev-buffers (oref strand :prev-buffers)))

    ;; clean the real list
    (set-window-prev-buffers window nil)
    ;; so the buffers we add go to the end of the list
    (save-window-excursion
      (with-selected-window window
        (cl-loop
              for buf in saved-prev-buffers
              do (when (and (buffer-live-p (car-safe buf))
                            (and predicate (funcall predicate (car buf)))
                            (not (or (eq (car buf) buffer)
                                     (alist-get (car buf) actual-prev-buffers))))
                   ;; rely on switch-to-buffer ?
                   ;; setting the prev-buffers imply
                   ;; to know the window-start and the window-point
                   (switch-to-buffer (car buf) nil t)))
        (cl-callf nconc actual-prev-buffers (window-prev-buffers window))
        (set-window-prev-buffers window nil)

        (when predicate
          (cl-loop
                for buf in (buffer-list)
                do (when (and (funcall predicate buf)
                              (not (or (alist-get buf actual-prev-buffers)
                                       (eq buf buffer))))
                     (switch-to-buffer buf nil t)))
          (cl-callf nconc actual-prev-buffers (window-prev-buffers window)))))

    ;; store the new prev buffer list
    (set-window-prev-buffers window actual-prev-buffers)
    (oset strand :prev-buffers actual-prev-buffers)))
