;;; moirai-eieio.el -*- lexical-binding: t; -*-

(defclass moirai-dired-side0 (moirai-strand)
  ((display-alist
    :initform '((moirai-dired-p)
                (display-buffer--in-dired-side-window)
                (slot . 0) (side . left) ;; (select . t)
                (preserve-size (t . t)) (window-width . 35)
                (body-function . (lambda (window) (select-window window)))
                (window-parameters . ((no-other-window . t)
                                      (no-delete-other-windows . t))))
    :custom (custom-variable-type 'display-buffer-alist)
    :documentation
    "Alist of conditionals actions for `display-buffer',
It is the same format that `display-buffer-alist'.")
   (dired-function
    :initform #'dired
    :documentation
    "The function called to create the buffer a dired side window.")
   ;; really necessary if I store the buffer ?
   (directory
    :initform default-directory
    :documentation
    "The actual directory of that dired side window.")
   (prev-buffers-predicate
    :initform #'moirai-dired-p
    :documentation
    "The predicate used to set the prev-buffer list of this dired side window.")
   (prev-buffers
    :initform nil
    :documentation
    "The previous buffers list of that dired side window.")
   (next-buffers
    :initform nil
    :documentation
    "The next buffers list of that dired side window.")))

(defun moirai-dired-p (buffer &optional _action)
  "Is this BUFFER using `dired-mode'?"
  (eq 'dired-mode
      (buffer-local-value 'major-mode (window-normalize-buffer buffer))))

;; >>> ???
(cl-defmethod morai-dired-buffer ((strand moirai-dired-side))
  (let* ((dir (oref strand :directory))
         (buffer (funcall (oref strand :dired-function)
                          (if (symbolp dir) (symbol-value dir) dir))))
    (oset strand :buffer buffer)))

;; (oref moirai-dired-side-left :directory)

;; (setq moirai-dired-side-left
;;       (moirai-dired-side))



;; (morai-dired-buffer moirai-dired-side-left)

(oref-default 'dired-side-window 'prev-buffers)

(morai-get-or-create 'moirai-dired-side 'moirai-dired-side-left)

;; (setf (frame-parameter (selected-frame) 'moirai-dired-side-left) nil)


(morai-build-prev-buffer-history (frame-parameter nil 'moirai-dired-side-left) w)
;; =>
;; ((#<buffer home>
;;            #<marker at 1 in home>
;;            #<marker at 93 in home>)
;;  (#<buffer user>
;;            #<marker at 1 in user>
;;            #<marker at 1147 in user>)
;;  (#<buffer src>
;;            #<marker at 1 in src>
;;            #<marker at 231 in src>))

(setf (moirai-ref 'moirai-dired-side-left :prev-buffers)
      nil)


(cl-print-object morai-dired-side standard-output)

(eieio-object-class 'morai-dired-side)

(eieio--class-p 'morai-dired-side)
;; if cl-union is a non-destructive operation
;; and cl-nunion is a destructive operation
;; how could cl-nunion call cl-union ?!

(defun display-buffer-in-dired-side-window (buffer alist)
  "Display BUFFER in a side window with `dired-side-window-parameters' as ALIST.

The side parameter that may be inherited from a selected dired side window.
Only apply for buffers using `dired-mode' without `inhibit-same-window'."
  (when (and (dired-buffer-p buffer)
             (not (cdr (assq 'inhibit-same-window alist))))
    (let* ((default-side (alist-get 'side (oref-default 'moirai-dired-side :display-alist)))
           (side (window-parameter (selected-window) 'window-side))
           (strand-name (intern (cl-reduce #'prin1-to-string
                                           `(morai-dired-side - ,(or side default-side)))))
           (strand (morai-get-or-create 'moirai-dired-side strand-name))
           (alist (cdr (oref strand :display-alist)))
           (window (display-buffer-in-side-window
                    buffer (if side
                               (cl-subst `(side . ,side) default-side alist)
                             alist))))
      (oset strand :buffer buffer)
      (oset strand :window window)
      (morai-update-prev-buffers strand window)
      window)))

;; (display-buffer-in-dired-side-window buf nil)


(oref-default 'moirai-dired-side0 'display-alist)
