;;; moirai-windmove.el -*- lexical-binding: t; -*-

;;; allow functions to be called with windmove-create-window
(advice-add #'windmove-do-window-select :override
            (defun windmove-do-window-select--side-window (dir &optional arg window)
              "Move to the window at direction DIR as seen from WINDOW.
DIR, ARG, and WINDOW are handled as by `windmove-find-other-window'.
If no window is at direction DIR, an error is signaled.
If `windmove-create-window' is non-nil, try to create a new window
in direction DIR instead."
              (let ((other-window (windmove-find-other-window dir arg window)))
                (when (and windmove-create-window
                           (or (null other-window)
                               (and (window-minibuffer-p other-window)
                                    (not (minibuffer-window-active-p other-window)))))
                  (setq other-window (if (functionp windmove-create-window)
                                         (funcall windmove-create-window dir window)
                                       (split-window window nil dir))))
                (cond ((null other-window)
                       (user-error "No window %s from selected window" dir))
                      ((and (window-minibuffer-p other-window)
                            (not (minibuffer-window-active-p other-window)))
                       (user-error "Minibuffer is inactive"))
                      (t
                       (select-window other-window))))))

;; need to respecify the defcustom
(setq windmove-create-window #'dired-side-windmove)

;;; fix the no-other-window thig
(advice-add 'windmove-find-other-window :override
            (defun windmove-find-other-window--side (dir &optional arg window)
              "Return the window object in direction DIR as seen from WINDOW.
DIR is one of `left', `up', `right', or `down'.
WINDOW must be a live window and defaults to the selected one.
Optional ARG, if negative, means to use the right or bottom edge of
WINDOW as reference position, instead of `window-point'; if positive,
use the left or top edge of WINDOW as reference point."
              (window-in-direction dir window windmove-ignore-no-other-window
                                   arg windmove-wrap-around t)))

(defcustom windmove-ignore-no-other-window t
  "Weither the windmove commands are allowed to target all type of windows,
If this variable is set to t, `windmove-find-other-window--side' and
subsequently all interactive windmove commandswill ignore the
no-other-window parameter.")

(setq windmove-ignore-no-other-window t)

;;; Now I can make things.
(defvar dired-side-windmove-sides '(left right)
  "")

(defun dired-side-windmove (dir &optional window)
  ""
  (let ((window (or window (selected-window)))
        (dir (pcase dir (`up 'top) (`down 'bottom) (_ dir))))
    (when (member dir dired-side-windmove-sides)
      (if (dired-side-window-p window)
          (and (window--delete window) nil)
        (let* ((dired-side-window-parameters
                (cl-subst `(side . ,dir) (assoc 'side dired-side-window-parameters)
                          dired-side-window-parameters))
               (window (display-buffer--in-dired-side-window
                        (dired-noselect ".")
                        dired-side-window-parameters)))
          (select-window window))))))

(provide 'moirai-windmove)
